@@ -15,7 +15,11 @@ MIT License see LICENSE for more details
 """
 
 import os
+from urllib import response
+from dotenv import load_dotenv
 import requests
+from requests.adapters import HTTPAdapter
+from urllib3.util.retry import Retry 
 from getwowdata import exceptions
 from getwowdata.urls import urls
 from getwowdata.helpers import get_id_from_url
@@ -61,11 +65,17 @@ class WowApi:
             wow_api_secret (str, optional): Your client secret from https://develop.battle.net/.
                 Ignore if secret is set as environment variable.
         """
+        retry = Retry(total=5, backoff_factor=0.1)
+        adapter = HTTPAdapter(max_retries=retry)
+        session = requests.Session()
+        session.mount('https://', adapter)
+        session.params = {'locale':locale}
+        self.session = session
         self.region = region
-        self.locale = locale
         self.wow_api_id = wow_api_id
         self.wow_api_secret = wow_api_secret
-        self.access_token = self._get_access_token()
+        access_token = self._get_access_token()
+        session.params['access_token'] = access_token
 
     def _get_access_token(
         self,
@@ -97,7 +107,7 @@ class WowApi:
 
         try:
             auth = (os.environ["wow_api_id"], os.environ["wow_api_secret"])
-            access_token_response = requests.post(
+            access_token_response = self.session.post(
                 urls["access_token"].format(region=self.region),
                 data=token_data,
                 auth=auth,
@@ -122,7 +132,7 @@ class WowApi:
                 ) from NameError
 
             auth = (self.wow_api_id, self.wow_api_secret)
-            access_token_response = requests.post(
+            access_token_response = self.session.post(
                 urls["access_token"].format(region=self.region),
                 data=token_data,
                 auth=auth,
@@ -182,6 +192,10 @@ class WowApi:
 
         Returns:
         A json looking dict with nested dicts and/or lists containing data from the API.
+        
+        Raises:
+            requests.exceptions.HTTPError: Raised on bad status code. Shows the problem causing error and url.
+            requests.exceptions.ConnectionError: Raised on network problem. 
         """
         try:
             timeout = extra_params.pop("timeout", 30)
@@ -191,17 +205,15 @@ class WowApi:
         conn_realm_search_params = {
             **{
                 "namespace": f"dynamic-{self.region}",
-                "access_token": self.access_token,
-                "locale": self.locale,
             },
             **extra_params,
         }
 
-        response = requests.get(
+        response = self.session.get(
             urls["search_realm"].format(region=self.region),
             params=conn_realm_search_params,
             timeout=timeout,
-        )
+        )       
         response.raise_for_status()
         return response.json()
 
@@ -249,6 +261,9 @@ class WowApi:
 
         Returns:
         A json looking dict with nested dicts and/or lists containing data from the API.
+        
+        Raises:
+            requests.exceptions.HTTPError: Raised on bad status code.  Shows the problem causing error and url.
         """
         try:
             timeout = extra_params.pop("timeout", 30)
@@ -258,16 +273,16 @@ class WowApi:
         search_params = {
             **{
                 "namespace": f"static-{self.region}",
-                "access_token": self.access_token,
             },
             **extra_params
         }
 
-        search_response = requests.get(
+        search_response = self.session.get(
             urls["search_item"].format(region=self.region),
             params=search_params,
             timeout=timeout,
         )
+        search_response.raise_for_status()
         return search_response.json()
 
     def get_connected_realms_by_id(
@@ -282,19 +297,24 @@ class WowApi:
 
         Returns:
             A json looking dict with nested dicts and/or lists containing data from the API.
+            
+        Raises:
+            requests.exceptions.HTTPError: Raised on bad status code.  Shows the problem causing error and url.
         """
         realm_params = {
             "namespace": f"dynamic-{self.region}",
-            "locale": self.locale,
-            "access_token": self.access_token,
+            
+            
         }
-        return requests.get(
+        response = self.session.get(
             urls["realm"].format(
                 region=self.region, connected_realm_id=connected_realm_id
             ),
             params=realm_params,
             timeout=timeout,
-        ).json()
+        )
+        response.raise_for_status()
+        return response.json()
 
     def get_auctions(self, connected_realm_id, timeout=30) -> dict:
         """Gets all auctions from a realm by its connected_realm_id.
@@ -307,19 +327,25 @@ class WowApi:
 
         Returns:
             A json looking dict with nested dicts and/or lists containing data from the API.
+            
+        Raises:
+            requests.exceptions.HTTPError: Raised on bad status code.  Shows the problem causing error and url.
         """
         auction_params = {
             "namespace": f"dynamic-{self.region}",
-            "locale": self.locale,
-            "access_token": self.access_token,
+            
+            
         }
-        return requests.get(
+        response = self.session.get(
             urls["auction"].format(
                 region=self.region, connected_realm_id=connected_realm_id
             ),
             params=auction_params,
             timeout=timeout,
-        ).json()
+        )
+        response.raise_for_status()
+        return response.json()
+
 
     def get_profession_index(self, timeout=30) -> dict:
         """Gets all professions including their names and ids.
@@ -331,17 +357,22 @@ class WowApi:
 
         Returns:
             A json looking dict with nested dicts and/or lists containing data from the API.
+            
+        Raises:
+            requests.exceptions.HTTPError: Raised on bad status code.  Shows the problem causing error and url.
         """
         prof_params = {
             "namespace": f"static-{self.region}",
-            "locale": self.locale,
-            "access_token": self.access_token,
+            
+            
         }
-        return requests.get(
+        response = self.session.get(
             urls["profession_index"].format(region=self.region),
             params=prof_params,
             timeout=timeout,
-        ).json()
+        )
+        response.raise_for_status()
+        return response.json()
 
     # Includes skill tiers (classic, burning crusade, shadowlands, ...) id
     def get_profession_tiers(self, profession_id, timeout=30) -> dict:
@@ -357,21 +388,25 @@ class WowApi:
 
         Returns:
             A json looking dict with nested dicts and/or lists containing data from the API.
+            
+        Raises:
+            requests.exceptions.HTTPError: Raised on bad status code.  Shows the problem causing error and url.
         """
         prof_tier_params = {
             "namespace": f"static-{self.region}",
-            "locale": self.locale,
-            "access_token": self.access_token,
+            
+            
         }
-        return requests.get(
+        response = self.session.get(
             urls["profession_skill_tier"].format(
                 region=self.region, profession_id=profession_id
             ),
             params=prof_tier_params,
             timeout=timeout,
-        ).json()
+        )
 
-    
+        response.raise_for_status()
+        return response.json()
 
     def get_profession_icon(self, profession_id, timeout=30) -> bytes:
         """Returns a profession's icon in bytes.
@@ -383,21 +418,24 @@ class WowApi:
 
         Returns:
             The profession's icon in bytes.
+            
+        Raises:
+            requests.exceptions.HTTPError: Raised on bad status code.  Shows the problem causing error and url.
         """
         prof_icon_params = {
             "namespace": f"static-{self.region}",
-            "locale": self.locale,
-            "access_token": self.access_token,
+            
+            
         }
-        icon_response = requests.get(
+        response = self.session.get(
             urls["profession_icon"].format(
                 region=self.region, profession_id=profession_id
             ),
             params=prof_icon_params,
             timeout=timeout,
-        ).json()
-
-        return requests.get(icon_response["assets"][0]["value"], timeout=timeout).content
+        )
+        response.raise_for_status()
+        return self.session.get(response.json()["assets"][0]["value"], timeout=timeout).content
 
     # Includes the categories (weapon mods, belts, ...) and the recipes (id, name) in them
     def get_profession_tier_categories(
@@ -416,13 +454,16 @@ class WowApi:
 
         Returns:
             A json looking dict with nested dicts and/or lists containing data from the API.
+            
+        Raises:
+            requests.exceptions.HTTPError: Raised on bad status code.  Shows the problem causing error and url.
         """
         prof_teir_recipe_params = {
             "namespace": f"static-{self.region}",
-            "locale": self.locale,
-            "access_token": self.access_token,
+            
+            
         }
-        return requests.get(
+        response = self.session.get(
             urls["profession_tier_detail"].format(
                 region=self.region,
                 profession_id=profession_id,
@@ -430,7 +471,9 @@ class WowApi:
             ),
             params=prof_teir_recipe_params,
             timeout=timeout,
-        ).json()
+        )
+        response.raise_for_status()
+        return response.json()
 
     def get_recipe(self, recipe_id, timeout=30) -> dict:
         """Returns a recipes details by its id.
@@ -442,16 +485,21 @@ class WowApi:
 
         Returns:
             A json looking dict with nested dicts and/or lists containing data from the API.
+            
+        Raises:
+            requests.exceptions.HTTPError: Raised on bad status code.  Shows the problem causing error and url.
         """
-        return requests.get(
+        response = self.session.get(
             urls["recipe_detail"].format(region=self.region, recipe_id=recipe_id),
             params={
                 "namespace": f"static-{self.region}",
-                "locale": self.locale,
-                "access_token": self.access_token,
+                
+                
             },
             timeout=timeout,
-        ).json()
+        )
+        response.raise_for_status()
+        return response.json()
 
     def get_recipe_icon(self, recipe_id, timeout=30) -> bytes:
         """Returns a recipes icon in bytes.
@@ -463,18 +511,21 @@ class WowApi:
 
         Returns:
             The recipe icon in bytes.
+            
+        Raises:
+            requests.exceptions.HTTPError: Raised on bad status code.  Shows the problem causing error and url.
         """
-        icon_response = requests.get(
+        response = self.session.get(
             urls["repice_icon"].format(region=self.region, recipe_id=recipe_id),
             params={
                 "namespace": f"static-{self.region}",
-                "locale": self.locale,
-                "access_token": self.access_token,
+                
+                
             },
             timeout=timeout,
-        ).json()
-
-        return requests.get(icon_response["assets"][0]["value"], timeout=timeout).content
+        )
+        response.raise_for_status()
+        return self.session.get(response.json()["assets"][0]["value"], timeout=timeout).content
 
 
     def get_item_classes(self, timeout=30) -> dict:
@@ -487,16 +538,21 @@ class WowApi:
 
         Returns:
             A json looking dict with nested dicts and/or lists containing data from the API.
+            
+        Raises:
+            requests.exceptions.HTTPError: Raised on bad status code.  Shows the problem causing error and url.
         """
-        return requests.get(
+        response = self.session.get(
             urls["item_classes"].format(region=self.region),
             params={
                 "namespace": f"static-{self.region}",
-                "locale": self.locale,
-                "access_token": self.access_token,
+                
+                
             },
             timeout=timeout,
-        ).json()
+        )
+        response.raise_for_status()
+        return response.json()
 
     # flasks, vantus runes, ...
     def get_item_subclasses(self, item_class_id, timeout=30) -> dict:
@@ -509,18 +565,23 @@ class WowApi:
 
         Returns:
             A json looking dict with nested dicts and/or lists containing data from the API.
+            
+        Raises:
+            requests.exceptions.HTTPError: Raised on bad status code.  Shows the problem causing error and url.
         """
-        return requests.get(
+        response = self.session.get(
             urls["item_subclass"].format(
                 region=self.region, item_class_id=item_class_id
             ),
             params={
                 "namespace": f"static-{self.region}",
-                "locale": self.locale,
-                "access_token": self.access_token,
+                
+                
             },
             timeout=timeout,
-        ).json()
+        )
+        response.raise_for_status()
+        return response.json()
 
     def get_item_set_index(self, timeout=30) -> dict:
         """Returns all item sets. Ex: teir sets
@@ -531,16 +592,21 @@ class WowApi:
 
         Returns:
             A json looking dict with nested dicts and/or lists containing data from the API.
+            
+        Raises:
+            requests.exceptions.HTTPError: Raised on bad status code.  Shows the problem causing error and url.
         """
-        return requests.get(
+        response = self.session.get(
             urls["item_set_index"].format(region=self.region),
             params={
                 "namespace": f"static-{self.region}",
-                "locale": self.locale,
-                "access_token": self.access_token,
+                
+                
             },
             timeout=timeout,
-        ).json()
+        )
+        response.raise_for_status()
+        return response.json()
 
     def get_item_icon(self, item_id, timeout=30) -> bytes:
         """Returns the icon for an item in bytes.
@@ -552,17 +618,21 @@ class WowApi:
 
         Returns:
             Item icon in bytes.
+            
+        Raises:
+            requests.exceptions.HTTPError: Raised on bad status code.  Shows the problem causing error and url.
         """
-        icon_response = requests.get(
+        response = self.session.get(
             urls["item_icon"].format(region=self.region, item_id=item_id),
             params={
                 "namespace": f"static-{self.region}",
-                "locale": self.locale,
-                "access_token": self.access_token,
+                
+                
             },
             timeout=timeout,
-        ).json()
-        return requests.get(icon_response["assets"][0]["value"], timeout=timeout).content
+        )
+        response.raise_for_status()
+        return self.session.get(response.json()["assets"][0]["value"], timeout=timeout).content
 
     def get_wow_token(self, timeout=30) -> dict:
         """Returns the price of the wow token and the timestamp of its last update.
@@ -575,16 +645,19 @@ class WowApi:
             A json looking dict with nested dicts and/or lists containing data from the API.
             The price is in the format g*sscc where g=gold, s=silver, and c=copper.
             Ex: 123456 = 12g 34s 56c
+            
+        Raises:
+            requests.exceptions.HTTPError: Raised on bad status code.  Shows the problem causing error and url.
         """
-        return requests.get(
+        response = self.session.get(
             urls["wow_token"].format(region=self.region),
             params={
                 "namespace": f"dynamic-{self.region}",
-                "locale": self.locale,
-                "access_token": self.access_token,
             },
             timeout=timeout,
-        ).json()
+        )
+        response.raise_for_status()
+        return response.json()
 
     def get_connected_realm_index(self, timeout=30) -> dict:
         """Returns a dict where {key = Realm name: value = connected realm id, ...}
@@ -595,6 +668,9 @@ class WowApi:
 
         Returns:
             A dict like {realm_name: connected_realm_id}
+            
+        Raises:
+            requests.exceptions.HTTPError: Raised on bad status code.  Shows the problem causing error and url.
         """
 
         index = {}
@@ -617,6 +693,11 @@ class WowApi:
 
         Returns:
              A json looking dict with nested dicts and/or lists containing data from raidbots.com
+        
+        Raises:
+            requests.exceptions.HTTPError: Raised on bad status code.  Shows the problem causing error and url.
         """
 
-        return requests.get(urls['item_bonuses']).json()
+        response = self.session.get(urls['item_bonuses'], params={'access_token': None, 'locale': None})
+        response.raise_for_status()
+        return response.json()
